/**
 * Copyright 2015 Holger Kaelberer <holger.k@elberer.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NOTIFYINGAPPLICATION_H
#define NOTIFYINGAPPLICATION_H

#include <QRegularExpression>

struct NotifyingApplication {
    QString name;
    QString icon;
    bool active;
    QRegularExpression blacklistExpression;

    bool operator==(const NotifyingApplication& other) const {
        return (name == other.name);
    }
};

Q_DECLARE_METATYPE(NotifyingApplication);

QDataStream& operator<<(QDataStream& out, const NotifyingApplication& app);
QDataStream& operator>>(QDataStream& in, NotifyingApplication& app);
QDebug operator<<(QDebug dbg, const NotifyingApplication& a);

#endif //NOTIFYINGAPPLICATION_H
